;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xd1e266a0ff2f61749fc84686849e80c5 . "./code.cg2js/cg-acache.fasl")
(#x828673fe7e8c2e4be559fa3d2f1e9052 . "./code.cg2js/cg-base.fasl")
(#xcfbb6e138c5947a6597fffa4ec54fb05 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x63f0f84915f6fe15ebb364784921ed22 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x50aee484633bdbf4979090f8eb043966 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x9d53a3f78fc14cf31e218bdb98a93ac3 . "./code.cg2js/cg-button.fasl")
(#xc255f27e7be478cab1b2ea0375215151 . "./code.cg2js/cg-calendar.fasl")
(#x20b25056be02ba5e4ae4ba8948087c74 . "./code.cg2js/cg-caret.fasl")
(#x7e88d7bc227cfc08c907bea34ab4dda2 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x462cf198c4954d1734c3bfeed3aaa083 . "./code.cg2js/cg-chart-widget.fasl")
(#x2130fe44c68083eec11fd9c6945dddfb . "./code.cg2js/cg-check-box.fasl")
(#xf3416f4fcace4893dc1e5ffdf45d0d66 . "./code.cg2js/cg-choice-list.fasl")
(#xb2dce94de6e8d6c9b7c6f405988b00b5 . "./code.cg2js/cg-choose-printer.fasl")
(#xa1a29506c41335e2c3aaf444c0b9ca3f . "./code.cg2js/cg-class-grid.fasl")
(#x1463bef8168612351bf865c8bb2c3c9b . "./code.cg2js/cg-class-slot-grid.fasl")
(#x7c7eb7fd9bbaa9ea8fa40f0f43eab78b . "./code.cg2js/cg-class-support.fasl")
(#x86be056cc71c9be2c22d2aafa17dee3e . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x9859f94deeba5f913dc0bffe21118b01 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x1d0560e431718fc2ffc75df9862a4392 . "./code.cg2js/cg-clipboard.fasl")
(#xd8bf34346915b7e6def941853ab75aeb . "./code.cg2js/cg-color-dialog.fasl")
(#xeb507cd68250fea0868e0577434c14ae . "./code.cg2js/cg-combo-box.fasl")
(#xe016fb65a5199fcd65f21ab9a291172a . "./code.cg2js/cg-common-control.fasl")
(#x57707c11db7b50f1a6d766c5e31869ec . "./code.cg2js/cg-comtab.fasl")
(#xd54e64820831b8c592e8a5c4fedcb62f . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x27d5a9bbed2c97fab37dcc512ae79fb7 . "./code.cg2js/cg-curve.fasl")
(#xa512e433e302d01e68fd03e967f9c2f7 . "./code.cg2js/cg-dde-utils.fasl")
(#x3ac83658b069ef4099c30e0958dfbcb4 . "./code.cg2js/cg-dev.fasl")
(#xf94393514bb8b742bb947ab4615161e3 . "./code.cg2js/cg-dialog-item.fasl")
(#x79268818b8f34063dccad4a0ab1c78ef . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x7ec06b1bb368192216a9e007776c3902 . "./code.cg2js/cg-directory-dialog.fasl")
(#x254fdb254b6e1bc60a949606a98dc7b1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x28d52d39f330d71adc3a835596415afa . "./code.cg2js/cg-drag-and-drop.fasl")
(#x607daea9494418b2cb1907b66ecb8948 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xc64d79e1a5b10d4a693579f66bdb71da . "./code.cg2js/cg-drawable.fasl")
(#x4cbe1b3d2b6c8522660a20f9424e01d8 . "./code.cg2js/cg-dropping-outline.fasl")
(#xf1cc04358dbc985dbb6d104cf89add08 . "./code.cg2js/cg-edit-in-place.fasl")
(#x5c557a76700fba9e75c972ceb3cff10d . "./code.cg2js/cg-editable-text.fasl")
(#xafc21dcd3405e6083d76de65a65dab61 . "./code.cg2js/cg-file-dialog.fasl")
(#x68e9b41192248c3aee7f61d2d21cf295 . "./code.cg2js/cg-fill-texture.fasl")
(#xa3455c5576af8b3768b49eadf2259b45 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x27d45914072807982c4eba7ac563a84f . "./code.cg2js/cg-font-dialog.fasl")
(#x92f23a803d7b87b133d954dd602de882 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x53c90e9614a1ab9ecdd2d0c61ae22271 . "./code.cg2js/cg-get-pixmap.fasl")
(#x0b95f22adf57c6b6f2db3c26744606c6 . "./code.cg2js/cg-get-position.fasl")
(#xf87d921459fedab1852c06c2453352d2 . "./code.cg2js/cg-graphics-context.fasl")
(#x28628dffa167744ea6205621c4076327 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xc12c0a458e225c9a18ff12c0651f1db8 . "./code.cg2js/cg-grid-widget.fasl")
(#xdbaf75920d2a8880cf964239edf477df . "./code.cg2js/cg-group-box.fasl")
(#xc35b69c4f8ec9247dd5d5dd7d7aa7b8f . "./code.cg2js/cg-header-control.fasl")
(#xecb7357a7726f0ee081c84b1d8747fe3 . "./code.cg2js/cg-hotspot.fasl")
(#x106f639390e33f3b05ad07a7391c568e . "./code.cg2js/cg-html-dialog.fasl")
(#xc4a3ddd09ce948f8c15027fa09124fda . "./code.cg2js/cg-html-widget.fasl")
(#x009fcdb2c691b3a1a93e3c8dcc0843b6 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xf8cd47f0f20c870e17c9892e52dadc4e . "./code.cg2js/cg-icon.fasl")
(#x14edfd7bf6ff44d465690d949368fcb3 . "./code.cg2js/cg-ie.fasl")
(#x83fd5c78778ddae931cf81e14d1ca48e . "./code.cg2js/cg-intersect-posbox.fasl")
(#x80a8871b28d992047272c28eba9ea05e . "./code.cg2js/cg-intersect.fasl")
(#xc3837fecb499e799e03c3c568d37e29b . "./code.cg2js/cg-item-list.fasl")
(#x60298329cbb58c66dbb2bf82af6c74f6 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x2c462cbf9663f2faeca3886649bd35d1 . "./code.cg2js/cg-lamp.fasl")
(#x1a64d21145d9e60e15b65306162a3586 . "./code.cg2js/cg-layout.fasl")
(#xb946638c09caae45e1463ec21bcc016e . "./code.cg2js/cg-lettered-menu.fasl")
(#x0c3bef55bdb3af27f2591af95326baca . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x0bfb7d52548238c24c2b657ee493ef2c . "./code.cg2js/cg-lisp-text.fasl")
(#xc63b5a7e9de7e5c65afc8aba8115b7ce . "./code.cg2js/cg-lisp-widget.fasl")
(#x933b2554182f6e041db91eca86f0401b . "./code.cg2js/cg-list-view.fasl")
(#x8ad09d2df89c9aefa96907d0d6552d1a . "./code.cg2js/cg-mci.fasl")
(#x69ca289a34ff0232d7f07727c4511ce9 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x3326247d47e24364628154e910c3cc39 . "./code.cg2js/cg-menu.fasl")
(#xc18c072bc2f25ada15984b080ea85de4 . "./code.cg2js/cg-message-dialog.fasl")
(#xe90b5aa53a544c72319e0514012e3910 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x7b4d79e9b2b9d072f17376bfb626d907 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5685f9d3c62a02a453fb7ff49ae9bba4 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xed1800778561cbdc909b0cbe7917246d . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xb84cf01185936a9e5003abafcf4a1ce1 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa3ac3ccd8cb4d714d15f1e7725bd750d . "./code.cg2js/cg-nodes.fasl")
(#x9bbeab52018984b3726351239751eb63 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xda676db6aa8923e1a1fe1e0fa8e8aa0d . "./code.cg2js/cg-object-editor.fasl")
(#x04bd2aaae43645f94f5d4bd8e3b0d1a5 . "./code.cg2js/cg-ocx.fasl")
(#x5f7cda11ce5f0fe021d0ebe00c1c5d15 . "./code.cg2js/cg-os-widget.fasl")
(#x7394b64aa94f6fdbad7b22ef737f30be . "./code.cg2js/cg-os-window.fasl")
(#x26069705fc5f9320b7f211687880d88d . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x72f41f40a5b7e5dffb9b02f0511647a3 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x133dcea5185d853d147fb5067dfb6539 . "./code.cg2js/cg-outline.fasl")
(#xb4cb179e26521d4b1a86f2c1d700a1c7 . "./code.cg2js/cg-palette.fasl")
(#xbf819e019503f68a80f7e5d3c02e6ad6 . "./code.cg2js/cg-paren-matching.fasl")
(#x5ce7a3d119a1303396ff27b045a7ed80 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc1216076e57ac203e29544e2e173a036 . "./code.cg2js/cg-picture-widget.fasl")
(#x69e7821f92070e080eb3b17cdb87bd3e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x0bfbf7d133e2dda5f408325b9e72fa7b . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd0ca8ac1de52984e89eb9284ae0b9afe . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xaaf833c80949f1c7fa587e929e71bd99 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbb5a52508f115f4e8cd1ff93abfaef4c . "./code.cg2js/cg-pixmap.fasl")
(#x1462606928ec9ce0c65eb5065d4f9f80 . "./code.cg2js/cg-player.fasl")
(#x4b72f0d95433717fe3b6dab4037cd9a6 . "./code.cg2js/cg-plot-widget.fasl")
(#x6ec80c43eb11fb689a8c30127579c6dc . "./code.cg2js/cg-printing.fasl")
(#x4b6afedbc89f1a962f0e0f945a5ef87e . "./code.cg2js/cg-progress-indicator.fasl")
(#xca4faecca96a7755115099a21c1ae680 . "./code.cg2js/cg-project-window.fasl")
(#xa9941d82b0802082eb3965439831e6c7 . "./code.cg2js/cg-property.fasl")
(#xadf78b970d37b2d04228fdaee3afb872 . "./code.cg2js/cg-radio-button.fasl")
(#x325278cc36ff2de8963a3c2653bb7c0a . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x21e7112fd36615c5e2e1a39b9a5cc54a . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x5679efe529b15b7eed057e8aa6e615a2 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x27b7d35bb0226b95b533a8ab2781d2dc . "./code.cg2js/cg-rich-edit.fasl")
(#x1568dc8d3d19575003ea282513b666ad . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa7e766d89a06d92e49ea82ea506d0c71 . "./code.cg2js/cg-scaling-stream.fasl")
(#x996c7971666dc233abdfbd0353c6b10c . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x48f3b3777f5a02eced65d8e89c035580 . "./code.cg2js/cg-scroll-bar.fasl")
(#xc928d3a3b42b31d29e6a063a949198ed . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xc3bf33428af64b1b09ea9dfa3ec38bbe . "./code.cg2js/cg-selected-object.fasl")
(#x7a1f4cd8bbcadab326fd39f2a9761037 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x46e2561e5e45fb9de302bf362bf62ee3 . "./code.cg2js/cg-split-bar.fasl")
(#xf67bfc0f7cdd2e3562388cf045645da5 . "./code.cg2js/cg-static-text.fasl")
(#xbc29c03aa9f2f9bb0cee29ff1e76f1b0 . "./code.cg2js/cg-status-bar.fasl")
(#xfbd9189dd23f222b6aa71557b1427fe3 . "./code.cg2js/cg-string-dialog.fasl")
(#xa38e3893d4472fb32d4c6c67dffb623c . "./code.cg2js/cg-tab-control.fasl")
(#x49baef0a4899b5fb12f836b145090716 . "./code.cg2js/cg-template-string.fasl")
(#x6e71d64ed9ca9a58bab320ea276ee6d0 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x85841a98294fd1af7af22f16273ab723 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x54f1bffb241107f94ebefadac24897a1 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xd6eae31e31c3d7ad987a171c99d56898 . "./code.cg2js/cg-text-or-combo.fasl")
(#xd17a6967d45aabd1b1a824bc65fcf254 . "./code.cg2js/cg-text-widget.fasl")
(#xf0e6bf77ba159717b33f6b5178683b0b . "./code.cg2js/cg-timer.fasl")
(#xd5da8dba32177c8576c0629d954e6c2c . "./code.cg2js/cg-toggling-widget.fasl")
(#x8da55e60f6abd5fab3e8f051ae52a19a . "./code.cg2js/cg-toolbar.fasl")
(#xd0c866a1ff95edfa73b6153c69831d86 . "./code.cg2js/cg-tooltip.fasl")
(#x09baee0d60f145c206c56ecdc5dfe464 . "./code.cg2js/cg-touch.fasl")
(#xe88de977ac098c7e5237f0ffd609d89f . "./code.cg2js/cg-trackbar.fasl")
(#x7bac4610da03b983145ea9225fb6a3c3 . "./code.cg2js/cg-tray.fasl")
(#x5caa78f56b87ac12885e7bed819d1c16 . "./code.cg2js/cg-up-down-control.fasl")
(#x54726e3999ee34597ed5bfa136e4b526 . "./code.cg2js/cg-utility-dialog.fasl")
(#xfa0bb55a86da8f3edf70ebc5402c9e2b . "./code.cg2js/cg-web-browser-dde.fasl")
(#x3a7176532068831150f827459801fc32 . "./code.cg2js/cg-web-browser.fasl")
(#xe6e4b463713b62b17b6ae908d8461dea . "./code.cg2js/cg-wrap-string.fasl")
(#xae0e8d9d394568c8fd93a3f788b7a759 . "./code.cg2js/cg-yes-no-list.fasl")
(#x8e2d366f3a352aa00b67a3f23b60755a . "./code.cg2js/cg.fasl")
(#x142f4b66d06b8c79ed10845467e3f284 . "./code.cg2js/cg-yes-no-string.fasl")
(#x84c895635ef29ef8b8178fb465f6d5df . "./code.idejs/ide-acache.fasl")
(#xcf7a63318a6287524ef30dd870176857 . "./code.idejs/ide-apropos.fasl")
(#xf1774d7e0f0ee2c105417c9434ab4b03 . "./code.idejs/ide-base.fasl")
(#xcd56cef24f3507465acc336001e625df . "./code.idejs/ide-class-browser.fasl")
(#xedb2429681a59c92181b25d162db024d . "./code.idejs/ide-class-grapher.fasl")
(#xe97aa3e64278cd6c94da334ae11a95b6 . "./code.idejs/ide-clipboard.fasl")
(#x23922f932fd89b5c266ea91bcad26ecf . "./code.idejs/ide-dev.fasl")
(#xd8a27e6f05edb128c976b13630ea7979 . "./code.idejs/ide-editor.fasl")
(#x94927272638b4c37b5291aa8165911f6 . "./code.idejs/ide-find-definitions.fasl")
(#xdb2d47919f81a1a6099be3bd050a9726 . "./code.idejs/ide-find-in-files.fasl")
(#xbc6ba4f45c5f42170b8897e018bf52ba . "./code.idejs/ide-form.fasl")
(#x1d0411af2ca2c9ee10eabe0463dd47f3 . "./code.idejs/ide-grapher.fasl")
(#x37839ed3633b5a87370bbc77fad46a13 . "./code.idejs/ide-inspector.fasl")
(#x30431be653932b98f107a8bbaa893515 . "./code.idejs/ide-intro.fasl")
(#x37c5df8e85872f76c1f39bb36599ee99 . "./code.idejs/ide-keyshort.fasl")
(#x40168344ae58c6fa297d32b168789e45 . "./code.idejs/ide-navigator.fasl")
(#xe703057c6ce27459e33a0981a97f9282 . "./code.idejs/ide-package-list.fasl")
(#xb9d804b8228bd633b13084e540c3cf07 . "./code.idejs/ide-patch-dialog.fasl")
(#xb38fa65067395b8931eebbe9cbb6939f . "./code.idejs/ide-process-browser.fasl")
(#xbf9fc0ed87fcffac6136e407e0b71ef1 . "./code.idejs/ide-project.fasl")
(#x9c2d22c8885c3fa6c738a96e983a7dfa . "./code.idejs/ide-trace.fasl")
(#x0d1402555858b7cda467590636781213 . "./code.idejs/ide-window-list.fasl")
(#x0ff7d4813617b242c334890d03199017 . "./code.idejs/ide.fasl")
(#xf954c7d1d43ea0e0cec9ada7b7316935 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
